using System;
using System.Collections;	// Hashtable, SortedList
using System.Xml;			// XmlTextParser

namespace OnlinePhotoShop
{
	/// <summary>
	/// Utility class providing helper methods to parse an XML
	/// product catalog file.
	/// </summary>
	/// <requirement>F:order_products</requirement>
	/// <requirement>F:products_options</requirement>
	public class ProductParser
	{
		/// <summary>
		/// Private member that stores the path of the XML product
		/// catalog file.
		/// </summary>
		private string mFilename;

		/// <summary>
		/// Constructor.
		/// </summary>
		/// <param name="filename">Path of XML product catalog
		/// file.</param>
		public ProductParser(string filename)
		{
			mFilename = filename;
		}

		/// <summary>
		/// Queries all products from a product catalog.
		/// </summary>
		/// <returns>SortedList where product identifier are stored in the Key and
		/// product names in the Value field.</returns>
		public SortedList ProductList()
		{
			SortedList table = new SortedList();
			XmlTextReader parser = new XmlTextReader(mFilename);

			// read all nodes
			while(parser.Read())
			{
				if (parser.NodeType == XmlNodeType.Element)
				{
					if (parser.Name == "product")
					{
						// found a product tag, add new item to list
						table.Add(parser.GetAttribute("id"), parser.GetAttribute("name"));

						// read the "inside" of the product tag
						// to advance to next product
						parser.ReadInnerXml();
					}
				}
			}

			// Found in unit tests: must close the file handle
			parser.Close();

			return table;
		}

		/// <summary>
		/// Queries all options of a given product from a product
		/// catalog.
		/// </summary>
		/// <param name="product">Product identifier who's available options
		/// will be returned.</param>
		/// <returns>SortedList of product options. Each entries Key field
		/// refers to the option identifier and the Value field refers to
		/// the option name plus price.</returns>
		public SortedList OptionList(string product)
		{
			SortedList table = new SortedList();
			XmlTextReader parser = new XmlTextReader(mFilename);

			// read all nodes
			while(parser.Read())
			{
				if (parser.NodeType == XmlNodeType.Element)
				{
					switch(parser.Name)
					{
						case "product":
							if (parser.GetAttribute("id") != product)
							{
								// different product,
								// "eat" up all its content
								parser.ReadInnerXml();
							} break;
						case "option":  
						{
							// option found, add to list
							double price =
								Convert.ToDouble(parser.GetAttribute("price"));
							table.Add(parser.GetAttribute("id"),
								parser.GetAttribute("name") + " - "
								+ price.ToString("C"));
						} break;
					}
				}
			}

			// Found in unit tests: must close the file handle
			parser.Close();

			return table;
		}

		/// <summary>
		/// Queries all properties of a given product from a product
		/// catalog excluding the product options. To query the product
		/// options use OptionList().
		/// </summary>
		/// <param name="product">Product identifier who's properties
		/// will be returned.</param>
		/// <returns>Hashtable of properties. The keys in the Hashtable
		/// refer to the tag names and the values to the text stored
		/// within the tag. The returned Hashtable also contains the
		/// product name attribute.</returns>
		public Hashtable ProductDetails(string product)
		{
			XmlTextReader parser = new XmlTextReader(mFilename);
			Hashtable table = new Hashtable();
			string field = "";

			// read all nodes
			while(parser.Read())
			{
				switch (parser.NodeType)
				{
					case XmlNodeType.Element:
					{
						switch(parser.Name)
						{
							case "product":  
							{
								// if not the wanted product read the inner xml
								// otherwise add product name to table
								if (parser.GetAttribute("id") != product)
									parser.ReadInnerXml();
								else
									table["name"] = parser.GetAttribute("name");
							}break;
							case "option":
							{
								// not interested in option tag
								parser.ReadInnerXml(); break;
							}
							default:
							{
								// store name of current tag
								field = parser.Name; break;
							}
						}
					} break;
					case XmlNodeType.Text:
					{
						// store the content using the tag name
						// as key in the Hashtable
						table[field] = parser.ReadString();
					} break;
				}
			}

			// Found in unit tests: must close the file handle
			parser.Close();

			return table;
		}

		#region Do It Yourself Chapter 11
		/// <summary>
		/// Queries all information of a given product option from a
		/// product catalog.
		/// </summary>
		/// <param name="product">Product identifier who's option is
		/// specified.</param>
		/// <param name="option">Option identifier who's properties
		/// will be returned.</param>
		/// <returns>Hashtable of the option's properties. The keys
		/// in the Hashtable refer to the tag names and the values to
		/// the text stored within the tag. The returned Hashtable
		/// also contains the option tag attributes name and price.
		/// </returns>
		public Hashtable OptionDetails(string product, string option)
		{
			XmlTextReader parser = new XmlTextReader(mFilename);
			Hashtable table = new Hashtable();
			string field = "";

			// read all nodes
			while(parser.Read())
			{
				switch (parser.NodeType)
				{
					case XmlNodeType.Element:
					{
						switch(parser.Name)
						{
							case "product":
							{
								// if not the wanted product read the inner xml
								if (parser.GetAttribute("id") != product) parser.ReadInnerXml();
							} break;
							case "option":	
							{
								// if not the wanted option also read the inner xml
								if (parser.GetAttribute("id") != option) 
								{
									parser.ReadInnerXml();
								}
								else
								{
									// option found, add name and price attribute
									table["name"] = parser.GetAttribute("name");
									table["price"] = parser.GetAttribute("price");
								}
							} break;
							default:
							{
								// store name of current tag
								field = parser.Name;
							} break;
						}
					} break;
					case XmlNodeType.Text:
					{
						// store the content using the tag name
						// as key in the Hashtable
						table[field] = parser.ReadString(); 
					} break;
				}
			}

			// Found in unit tests: must close the file handle
			parser.Close();

			return table;
		}
		#endregion
	}
}
